function [x,r,nsteps,edges] = acl_method_1(P,v,d,alpha,tol)
% ACL_METHOD Solve PageRank via the ACL method
%
% [x,r] = acl_method_1(P,v,d,alpha,tol,rho)
%   finds x such that 0 <= v + alpha*P* x - x <= tol*d
% where each step involves increaing x(j) by r(j)-rho*d(j)*tol.
% and is specialized to the case of setting rho = 1, which results in the
% sparsest vector possible
%
% When rho = 1, then the method also solves a particular 1-norm regularized
% problem.

% This is a really crappy implementation of Reid's algorithm
% for testing

n = size(P,1);
x = zeros(n,1);
r = full(v);
d = full(d);
r1 = zeros(n,1); % this is the piece of the residual larger than tol*d

inQ = zeros(n,1);
Q = zeros(n,1);
qhead = 1;
qtail = 1;

for i = find(r)'
    if r(i) > d(i)*tol 
        inQ(i) = 1;
        Q(qtail) = i;
        qtail = mod(qtail,n) + 1;
        r1(i) = r(i) - d(i)*tol;
    end
end

Pt = P';
iter = 1;
nedges = 0;
tolf = (1+eps(1))*tol;

while qtail-qhead + n*(qtail < qhead) > 0
    j = Q(qhead);
    qhead = mod(qhead,n) + 1;
    inQ(j) = 0;
    
    incr = r1(j);
    
    assert(incr > 0);
    x(j) = x(j) + incr;
    r(j) = d(j)*tol;
    r1(j) = 0;
    for i = find(Pt(:,j))'
        assert(i ~= j);
        nedges = nedges + 1;
        rold = r(i);
    	r(i) = r(i) + alpha*incr*Pt(i,j);
        
        if ~inQ(i) && r(i) > d(i)*tolf
            r1(i) = max(alpha*incr*Pt(i,j) - (d(i)*tol-rold), 0);
            if r1(i) == 0, continue; end
            inQ(i) = 1;
            Q(qtail) = i;
            qtail = mod(qtail,n) + 1;
            assert(qtail ~= qhead); 
        elseif inQ(i) 
            r1(i) = r1(i) + alpha*incr*Pt(i,j);
        end
    end
    iter = iter + 1;
    if mod(iter,n) == 0
        fprintf('step = %4i, sumr = %9.3e, qlen = %8i\n', ...
            iter, sum(r), qtail-qhead + n*(qtail < qhead));
    end
end

nsteps = iter;
edges = nedges;
