%% Use the USPS data for semi-supervised learning
% This experiment replicates the one done in Zhou 2003 on semi-supervised
% learning on a graph.

%% Load the data
load data/usps.mat

%% digit subset
digits = [1,2,3,4]+1; % handle the offset that 1 = 0
patterns = [];
labels = [];
for d = digits
    filt = train_labels(d,:)==1;
    labels = [labels d*ones(1,sum(filt))];
    patterns = [patterns train_patterns(:,filt)];
    filt = test_labels(d,:)==1;
    labels = [labels d*ones(1,sum(filt))];
    patterns = [patterns test_patterns(:,filt)];
end
labels = labels-1; % handle the offset 

%% construct the graph
D = pdist2(patterns',patterns','euclidean');
%%
rad = 1.25;
K = exp(-D.^2/(2*rad^2));
G = K - diag(diag(K));
n = size(K,1);
d = sum(G,2);
W = diag(1./sqrt(d))*G*diag(1./sqrt(d));
P = G*diag(1./d);
%%
beta = 0.99;
% R = chol(K) => R'*R = K, so Kx = Y => R'*Rx = Y => Rx = (R'\Y)
K = eye(n) - alpha*W; R = chol(K); iK1 = @(Y) R \ (R' \ Y); 
% P*A = L*U => A = P'*L*U
% P'*L*U x = Y => L*U x = P*Y => x = U\
K = eye(n) - alpha*P; [L,U,Perm] = lu(K); iK2 = @(Y) U \ (L \ (P*Y));
K = diag(d) - alpha*G; R = chol(K); iK3 = @(Y) R \ (R' \ Y); 
%%
ntrials = 100;
nlabeled = [4 10:5:50];
nl = numel(nlabeled);
classes = unique(labels);
nclasses = numel(classes);

assert(numel(labels) == n);
errrate = zeros(nl, ntrials, 3);
for j=1:nl
    nlabels = nlabeled(j)
    for i=1:ntrials
        %%
        % pick the training set, it needs to have one element from each class.
        p = randperm(n);
        train_set = zeros(n,1);
        pl = labels(p);
        [cp,ia] = unique(pl,'first'); %pl(ia) = c
        train_set(p(ia)) = 1;
        test_set = ~train_set;
        test_labels = labels(test_set)';


        p(ia) = [];
        train_set(p(1:(nlabels - nclasses))) = 1;

        Y = full(sparse(find(train_set),labels(logical(train_set)),1,n,nclasses));
        % Todo, optimize the solves

        Pred = iK1(Y);
        [~,Pi] = max(Pred,[],2); % find the largest entry in each row
        pred_labels = Pi(test_set);
        errs = numel(test_set) - sum(pred_labels == test_labels);
        errrate(j,i,1) = errs/numel(test_set);

        Pred = iK2(Y);
        [~,Pi] = max(Pred,[],2); % find the largest entry in each row
        pred_labels = Pi(test_set);
        errs = numel(test_set) - sum(pred_labels == test_labels);
        errrate(j,i,2) = errs/numel(test_set);
        
        Pred = iK3(Y);
        [~,Pi] = max(Pred,[],2); % find the largest entry in each row
        pred_labels = Pi(test_set);
        errs = numel(test_set) - sum(pred_labels == test_labels);
        errrate(j,i,3) = errs/numel(test_set);
    end
end

%%
plot(squeeze(mean(errrate,2)),'o-');
legend('Kernel 1', 'Kernel 2', 'Kernel 3');