%% Use the USPS data for semi-supervised learning and show a single-class ROC curve

%% Load the data
load data/usps.mat
solve_cvx = 0; % = 0 means to use gurobi, = 1 means use (SLOW) cvx

%% digit subset
digits = [1,2,3,4]+1; % handle the offset that 1 = 0
patterns = [];
labels = [];
for d = digits
    filt = train_labels(d,:)==1;
    labels = [labels d*ones(1,sum(filt))];
    patterns = [patterns train_patterns(:,filt)];
    filt = test_labels(d,:)==1;
    labels = [labels d*ones(1,sum(filt))];
    patterns = [patterns test_patterns(:,filt)];
end
labels = labels-1; % handle the offset 

%% construct the graph
D = pdist2(patterns',patterns','euclidean');
rad = 2.5;
K = exp(-D.^2/(2*rad^2));
Gf = K - diag(diag(K));
G = Gf;
%G(G<exp(-10)) = 0; % determined this threshold by hand
G = sparse(G);
n = size(K,1);
d = sum(G,2);
% Patch up the empty rows with self-loops
%G(d==0,d==0) = eye(sum(d==0));
% Patch up empty rows with full sets (this seems critical to the last bit
% of performance)
G(:,d==0) = Gf(:,d==0);
G(d==0,:) = Gf(d==0,:);
%% Try a knn constructions
n = size(patterns,2);
k = 50;
rad = 1.25;
[Dm,I] = pdist2(patterns',patterns','euclidean','Smallest',k);
J = repmat((1:n),k,1);
G = sparse(I(:),J(:),exp(-Dm(:).^2/(2*rad^2)),n,n);
G = max(G,G');
G = G - diag(diag(G));
%%
d = sum(G,2);
W = diag(1./sqrt(d))*G*diag(1./sqrt(d));
P = G*diag(1./d);
dw = sum(W,2);
%%
beta = 0.99;
alpha = (1-beta)/beta;
% R = chol(K) => R'*R = K, so Kx = Y => R'*Rx = Y => Rx = (R'\Y)
K = full(eye(n) - beta*W); R = chol(K); iK1 = @(Y) R \ (R' \ Y); 
K = full(diag(d) - beta*G); R = chol(K); iK2 = @(Y) R \ (R' \ Y); 
K = full(diag(dw) - beta*W); R = chol(K); iK3 = @(Y) R \ (R' \ Y); 

%% Just predict one class.

% form the train/test set
nlabels = 15;
classes = unique(labels);
nclasses = numel(classes);

p = randperm(n);
train_set = zeros(n,1);
pl = labels(p);
[cp,ia] = unique(pl,'first'); %pl(ia) = c
train_set(p(ia)) = 1;
test_set = ~train_set;
test_labels = labels(test_set)';

test_filt = @(x) x(test_set);

p(ia) = [];
train_set(p(1:(nlabels - nclasses))) = 1;

Y = full(sparse(find(train_set),labels(logical(train_set)),1,n,nclasses));
c = 2;
y = Y(:,c);
s = Y(:,c);

tau = 1e-4*sum(s>0);

rocy = {};
rocy{end+1} = test_filt(iK1(y));
rocy{end+1} = test_filt(iK2(y));
rocy{end+1} = test_filt(iK3(y));
%rocy{end+1} = test_filt(prcut_gurobi(G,alpha,s,d));
rocy{end+1} = test_filt(prl1_gurobi(W,alpha,(s>0).*dw,dw,1e-5));
%%
clf; hold all;
for i=1:numel(rocy)
    [px,py] = perfcurve(test_labels, rocy{i},c);
    plot(px,py);
end
hold off;
legend