%% The Amazon study.
% In this case, we are only looking at ROC curves because the data for
% Amazon is an overlapping clustering. Our goal is to predict products in
% the same hierarchy. 
%
% We take the amazon dataset, and the top5000 categories as determined
% by Leskovec.
%
% We compare:
%   Zhou et al's semi-supervised learning with alpha=0.99
%   Our l1-regularized version of Zhou using the vector from the best
%   conductance set.
%   We'll show ROC curves for
%     Amazon
%     Amazon + 1 step of completion
%     Amazon + 2 steps of completion

amazon = load_matfile_data('amazon');

%%
Csize = sum(amazon.C);
C = amazon.C(:,Csize >= 50);
[~,p] = sort(sum(C),'descend');
C = C(:,p);

%% parameters
nlabels = 20;
nerrs = 10;
ntrials = 5;
beta = 0.99;
alpha = (1-beta)/beta;

%% Predict the error
A = amazon.A;
c = 25; % pick the cluster
G = (A + A*A + A*A*A);
G = G - diag(diag(G));
d = sum(G,2);
n = size(G,1);

Comm = C(:,c);
cverts = find(Comm);
L = sparse(n,ntrials);
% Run all of Zhou at once.
for i=1:ntrials
    Comm = C(:,c);
    p = randperm(numel(cverts));
    Labels = [cverts(p(1:nlabels)); randi(n,nerrs,1)];
    L(Labels,i) = 1;
end

tic
Y = zhou2solve(G,d,beta,full(L));
toc
[~,R] = ssl_cut_round(Y,A,'ncut');
prec = sum(spones(isfinite(R(cverts,:))))./sum(isfinite(R));
recall = sum(spones(isfinite(R(cverts,:))))/numel(cverts);
f1 = mean(2*prec.*recall/(prec + recall))

%%
tic
Y2 = zeros(n,ntrials);
for i=1:ntrials
    s = L(:,i);
    fprintf('trial %i\n', i);
    xt = prl1_fast(G,alpha,s./d,d,0.00000001);
    Y2(:,i) = xt; 
end
toc
[~,R] = ssl_cut_round(Y2,A,'ncut');
prec = sum(spones(isfinite(R(cverts,:))))./sum(isfinite(R));
recall = sum(spones(isfinite(R(cverts,:))))/numel(cverts);
f1 = mean(2*prec.*recall/(prec + recall))

%%
clf; hold on;
for i=1:ntrials
    [px,py] = perfcurve(full(Comm),Y(:,i),1);
    semilogy(px,py,'-')
    drawnow;
    
    [px,py] = perfcurve(full(Comm),Y2(:,i),1);
    semilogy(px,py,'-','Color','red')
    drawnow;
end




    