%% Vary the density of the amazon dataset and see the changes due to regularization

% We can vary the following
%
% number of labels
% number of trials - fix at 25?, each trial takes 3-5 minutes ...
% number of errors
% number of clusters
%

nlabels = [5,10,15,20];
nerrs = [1,1,2,2];
nclusters = 20; % Kloumann and Kleinberg say we should use 20 here
ntrials = 20;
ks = [1,2,3];

%nlabels = [5,20];
%nerrs = [1,2];
%nclusters = 3; % Kloumann and Kleinberg say we should use 20 here
%ntrials = 3;
%ks = [1,3];

beta = 0.99;
alpha = (1-beta)/beta;
kappa = 0.000001;

amazon = load_matfile_data('amazon');

n = size(amazon.A,1);

rng(1);

%% Get the community subset that Kloumann and Kleinberg advocate.
Csize = sum(amazon.C);
maxc = max(Csize);
Cdiff = abs(Csize - maxc^(3/4));
[~,p] = sort(Cdiff);
C = amazon.C(:,p(1:nclusters));

%% Run the experiments
prec_results = zeros(numel(ks),nclusters,numel(nlabels),ntrials);
recall_results = prec_results;
reg_prec_results = prec_results;
reg_recall_results = prec_results;
t0 = tic;
for ki=1:numel(ks)
    k = ks(ki);
    rng(k);
    G = amazon.A + speye(n);
    for i=2:k
        G = amazon.A*G + speye(n);
    end
    G = G - diag(diag(G));
    d = sum(G,2);
    
    for ci=1:nclusters
        Comm = C(:,ci);
        cverts = find(Comm);
        
        L = sparse(n,ntrials*numel(nlabels));
        
        % Run all of Zhou at once. for nlabels*ntrials
        
        runi=1;
        for li=1:numel(nlabels)
            nl = nlabels(li);
            ne = nerrs(li);
            
            for ti=1:ntrials
                p = randperm(numel(cverts));
                Labels = [cverts(p(1:nl)); randi(n,ne,1)];
                L(Labels,runi) = 1;
                runi = runi+1;
            end
        end
        
        Y = zhou2solve(G,d,beta,full(L));
        Y2 = zhou2solve_reg(G,d,beta,full(L),kappa);
        
        [~,R] = ssl_cut_round(Y,G,'ncut');
        prec = sum(spones(isfinite(R(cverts,:))))./sum(isfinite(R));
        recall = sum(spones(isfinite(R(cverts,:))))/numel(cverts);
        
        runi=1;
        for li=1:numel(nlabels)
            for ti=1:ntrials
                prec_results(ki,ci,li,ti) = prec(runi);
                recall_results(ki,ci,li,ti) = recall(runi);
                runi = runi+1;
            end
        end
        
        [~,R] = ssl_cut_round(Y2,G,'ncut');
        prec = sum(spones(isfinite(R(cverts,:))))./sum(isfinite(R));
        recall = sum(spones(isfinite(R(cverts,:))))/numel(cverts);
        
        runi=1;
        for li=1:numel(nlabels)
            for ti=1:ntrials
                reg_prec_results(ki,ci,li,ti) = prec(runi);
                reg_recall_results(ki,ci,li,ti) = recall(runi);
                runi = runi+1;
            end
        end
        fprintf('%8.1f sec - k=%i - Finished cluster %2i of %2i\n', toc(t0), ...
            k, ci, nclusters);

		save 'amazon-results-temp.mat' *results* alpha beta nlabels ks nerrs ntrials
    end
    
    f1 = 2*prec_results.*recall_results./(prec_results + recall_results);
    reg_f1 = 2*reg_prec_results.*reg_recall_results./(reg_prec_results + reg_recall_results);
    f1(~isfinite(f1)) = 0;
    reg_f1(~isfinite(reg_f1)) = 0;

    fprintf('Done with k=%i\n', k);
    fprintf('%10s  %6s  %6s\n', 'num labels', 'no reg', 'reg');
    fprintf('%10s  %6s  %6s\n', '----------', '------','------');
    for li=1:numel(nlabels)
        scorevec = f1(ki,:,li,:);
        score = mean(scorevec(:));
        reg_scorevec = reg_f1(ki,:,li,:);
        reg_score = mean(reg_scorevec(:));
        
        fprintf('%10i  %6.4f  %6.4f\n', nlabels(li), score, reg_score);
    end
end

save 'amazon-results-all.mat' *results* alpha beta nlabels ks nerrs ntrials

%% Get the results
f1 = 2*prec_results.*recall_results./(prec_results + recall_results);
reg_f1 = 2*reg_prec_results.*reg_recall_results./(reg_prec_results + reg_recall_results);

for ki=1:numel(ks)
    for li=1:numel(nlabels)
        scorevec = f1(ki,:,li,:);
        score = mean(scorevec(:));
        reg_scorevec = reg_f1(ki,:,li,:);
        reg_score = mean(reg_scorevec(:));
    end
end
        
    