function Y = zhou2solve(W,d,beta,L)
% ZHOU2Solve Solve the zhou2 diffusion from L
%
% We solve the Zhou 2 diffusion:
%  (D - beta*W) x = f
% up to machine precision where n is the size of A.
rhsnorm = max(sum(L));
n = size(W,1);

tol = eps(1)*rhsnorm;

maxsteps = log(tol)/log(beta);
iD = repmat(1./d(:), 1, size(L,2));
Y = iD.*L;
for i=1:maxsteps
    Ynew = (L + beta*(W'*Y)).*iD;
    R = Ynew - Y;
    err = (1/(1-beta))*max(max(abs(R)));
    Y = Ynew;
    if err < tol
        break;
    end
end

% Notes
% use the Jacobi iteration
%  (D - beta*W) x = f
% D*x(k+1) = f + beta*W*x(k)
%  note that at each step
% D*x = f + beta*W*x
%  ||err(k+1)||_infty <= beta*err(k+1)_infty
% But also, 
%  (D - beta*W) err = resid
%  D*(I - beta*D^{-1}*W)err = resid
%  I - beta*D^{-1}*W) err = D^{-1} resid
%  ||err
%  ||err|| <= ||(I-beta*D^{-1}*W)^{-1}||_inf || D^{-1} resid||_inf
%  
%
% So if we solve (I - beta D^{-1} W) Y = D^{-1} L
% then the residual is Y - beta*D^{-1}*W*Y - D^{-1} L

