function Y = zhou2solve(A,d,beta,L,kappa)
% ZHOU2Solve_REG Solve the regularized zhou2 diffusion from L
%
% See prl1_fast


t1 = tic;
t0 = tic;
dt1 = 5;
print1 = 1;
flag = 1;

d = full(d(:));
D = repmat(d,1,size(L,2));
volS = sum(D.*L);
V = (D.*L)*diag(1./volS);

taus = kappa*beta./volS;

tD = D*diag(taus);

%x = pprfast(A,d,beta,v,tau,'tol',sqrt(eps(1)),'skip_push',true);
%x = pprfast(A,d,beta,v,tau,'tol',sqrt(eps(1)),varargin{:});

tol = sqrt(eps(1));

n = size(A,1);
X = zeros(size(V));
R = (1-beta)*V;
S = tD - R;
curiter = true(1,size(V,2));

    maxit = 10*n; % Kyle has a better bound here.

    Rp = max(R - tD,0);

    for i=1:maxit
        X = X + Rp;
        R = (1-beta)*V + beta*(A'*(X./D)) - X;
        Rp = max(R - tD,0);
        S = tD - R;
        dt = toc(t1);
        if dt > dt1
            if print1 % print the header
                fprintf('  %5s  %11s  %11s  %11s  %8s\n', ...
                    'iter', 'sum rp', 'dual', 'sum x', 'time');
                fprintf('  -----  -----------  -----------  -----------  --------\n');
            end
            fprintf('  %5i  %11.3e  %11.3e  %11.3e  %8.1f\n', ...
                i, max(sum(Rp)), max(abs(diag(X'*S))/n), min(sum(X)), toc(t0));
            t1 = tic;
            print1 = 0;
        end
        
        curiter = (abs(diag(X'*S))/n >= tol) | (sum(Rp)' >= tol);
        
        if ~any(curiter)
            flag = 0;
            break;
        end
    end
    
    if flag == 0
        if print1 == 0
            % final print
            fprintf('* %5i  %11.3e  %11.3e  %11.3e  %8.1f\n', ...
                i, max(sum(Rp)), max(abs(diag(X'*S))/n), min(sum(X)), toc(t0));
        end
    end

Y = (X./D)*diag(volS);