nlabels = 3;
beta = 0.99;
alpha = (1-beta)/beta;
kappa = 1e-5;
blocksizes = [30,10,20];

rng(1);
[A,labels] = bigblock(blocksizes,[0.7,0.9,0.8],0.05);
xy = block_layout(blocksizes);
A = A - diag(diag(A));
d = full(sum(A,2));
n = size(A,1);
nclasses = numel(blocksizes);

[test_set,test_labels,train_set] = build_test_train(labels,nlabels);
Y = full(sparse(find(train_set),labels(logical(train_set)),1,n,nclasses));

%%
imagesc(A); colormap(1-gray); %colormap([1,1,1;0,0,0;0,0.8,0]);

axis square;
axis off;

curoffset = 1;
for b = 1:numel(blocksizes)
    bs = blocksizes(b);
    start = curoffset - 0.5;
    line([start,start+bs,start+bs,start,start], ...
         [start,start,start+bs,start+bs,start],'Color',[0.8,0,0]);
    curoffset = curoffset + bs;
end
set(gca, 'Position', get(gca, 'OuterPosition') - ...
    get(gca, 'TightInset') * [-1 0 1 0; 0 -1 0 1; 0 0 1 0; 0 0 0 1]);
set_figure_size([3,3]);
print('clean-example-matrix.eps','-depsc2');

%%
close all;

%%

tic;
Pred = zeros(size(Y));
for c=1:nclasses
    s = Y(:,c);
    assert(all(s >= 0));
    s = sparse(s);
    
    zt = prl1_fast(A,alpha,s,d,0);
    
    Pred(:,c) = zt;
end
toc
prediction_plot(Pred,blocksizes);

%%
tic;
Pred = zeros(size(Y));
for c=1:nclasses
    s = Y(:,c);
    assert(all(s >= 0));
    s = sparse(s);
    
    prl1 = @(kappa) prl1_fast(A,alpha,s,d,kappa);
    zt = prl1_sweep(prl1,0.75,0.5);
    
    Pred(:,c) = zt;
end
toc
prediction_plot(Pred,blocksizes);
legend off;
print('clean-flow-improve-3-regularized.eps','-depsc2');


%%
Ys = (diag(d) - beta*A)\(diag(d)*Y); prediction_plot(Ys,blocksizes);
legend off;
print('clean-flow-improve-3.eps','-depsc2');
%%
[~,Ys] = zhou_ssl_1(A,Y); prediction_plot(Ys,blocksizes);
%%
[~,Ys] = zhou_ssl_2(A,Y); prediction_plot(Ys,blocksizes);
set(legend,'Location','Northwest');
print('clean-zhou2-3.eps','-depsc2');
%%
[~,Ys] = joachims_ssl(A,Y); prediction_plot(Ys,blocksizes);
set(legend,'Location','Northwest');
legend off;
print('clean-joachims-3.eps','-depsc2');
%%
[~,Ys] = zgl_ssl(A,Y); prediction_plot(Ys,blocksizes);
set(legend,'Location','West');
legend off;
print('clean-zgl-3.eps','-depsc2');

%%
nlabels = 15;
rng(1);
[A,labels] = bigblock(blocksizes,[0.7,0.9,0.8],0.05);
xy = block_layout(blocksizes);
A = A - diag(diag(A));
d = full(sum(A,2));
[test_set,test_labels,train_set] = build_test_train(labels,nlabels);
Y = full(sparse(find(train_set),labels(logical(train_set)),1,n,nclasses));
Yn = normout(Y')';

%% 
Ys = (diag(d) - beta*A)\(diag(d)*Y); prediction_plot(Ys,blocksizes);
set(legend,'Location','West');
legend off;
print('clean-flow-improve-15.eps','-depsc2');

%%
tic;
Pred = zeros(size(Y));
for c=1:nclasses
    prl1 = @(kappa) prl1_fast(A,alpha,sparse(Y(:,c)),d,kappa);
    zt = prl1_sweep(prl1,0.75,0.5);
    Pred(:,c) = zt;
end
toc
prediction_plot(Pred,blocksizes);
legend off;
print('clean-flow-improve-15-regularized.eps','-depsc2');


%%
[~,Ys] = zhou_ssl_2(A,Y); prediction_plot(Ys,blocksizes);
set(legend,'Location','Northwest');
legend off;
print('clean-zhou2-15.eps','-depsc2');
%%
[~,Ys] = joachims_ssl(A,Y); prediction_plot(Ys,blocksizes);
set(legend,'Location','West');
legend off;
print('clean-joachims-15.eps','-depsc2');
%%
[~,Ys] = zgl_ssl(A,Y); prediction_plot(Ys,blocksizes);
set(legend,'Location','West');
legend off;
print('clean-zgl-15.eps','-depsc2');

%%
%[~,Ys] = zhou_ssl_2(A,Y);
Ys = (diag(sum(A,2)) - 0.99*A) \ Yn;
prediction_plot(Ys,blocksizes); 
legend off;
print('clean-zhou2-15-norm.eps','-depsc2');
%%
rng(1);
E = spones(sprand(n,n,2/n)); E = triu(E,1); E = E + E';
C = max(A,E);
Ys = (diag(sum(C,2)) - 0.99*C) \ Yn;
prediction_plot(Ys,blocksizes); 

%%
trainl = labels(logical(train_set));
trainl(5) = 2;
Z = full(sparse(find(train_set),trainl,1,n,nclasses));
Zn = normout(Z')';
Ys = (diag(sum(C,2)) - 0.99*C) \ Zn;
prediction_plot(Ys,blocksizes); 

%%
%print('clean-zhou2-15-norm-error.eps','-depsc2');