nlabels = 3;
beta = 0.99;
alpha = (1-beta)/beta;
kappa = 1e-5;
blocksizes = [32,16,8];

rng(1);
[A,labels] = bigblock(blocksizes,[1,1,1],2/sum(blocksizes));
xy = block_layout(blocksizes);
A = A - diag(diag(A));

d = full(sum(A,2));
n = size(A,1);
classes = unique(labels);
nclasses = numel(classes);

p = randperm(n);
train_set = zeros(n,1);
pl = labels(p);
[cp,ia] = unique(pl,'first'); %pl(ia) = c
train_set(p(ia)) = 1;
test_set = ~train_set;
test_labels = labels(test_set)';

p(ia) = [];
train_set(p(1:(nlabels - nclasses))) = 1;

Y = full(sparse(find(train_set),labels(logical(train_set)),1,n,nclasses));
Yt = diag(1./sqrt(d))*Y; 

tic;
Pred = zeros(size(Y));
for c=1:nclasses
    s = Y(:,c);
    assert(all(s >= 0));
    s = sparse(s);
    
    
    pralpha = 1/(1+alpha);
    volS = sum(d.*s);
    v = (d.*s)/volS;
    v = sparse(v);
    d = full(d);
    A = sparse(A);

    tau = kappa*beta/volS;
    %[x,r,s] = pprfast(A, d, beta, v, tau, 'tol',sqrt(eps(1)));
    x = (diag(d) - beta*A)\((1-beta)*v);
    x = x.*d;
    zt = (x./d)*volS;
    
    Pred(:,c) = zt;
end
toc
plot(Pred)

%%
Ys = (diag(d) - beta*A)\(diag(d)*Y); plot(Ys)

%%
[~,Ys] = zhou_ssl_1(A,Y); plot(Ys)
%%
[~,Ys] = zhou_ssl_2(A,Y); plot(Ys)
%%
[~,Ys] = joachims_ssl(A,Y); plot(Ys)
%%
[~,Ys] = zgl_ssl(A,Y); plot(Ys)
%% Try with sorting
Ys = (diag(d) - beta*A)\(diag(d)*Y);
[~,R] = ssl_rank_round(Ys); plot(R)

%%
[~,Ys] = zhou_ssl_1(A,Y); plot(sort(Ys))
[~,R] = ssl_rank_round(Ys); plot(R)
%%
[~,Ys] = zhou_ssl_2(A,Y); plot(Ys)
[~,R] = ssl_rank_round(Ys); plot(R)
%%
[~,Ys] = joachims_ssl(A,Y); plot(Ys)
[~,R] = ssl_rank_round(Ys); plot(R)
%%
[~,Ys] = zgl_ssl(A,Y); plot(Ys)
[p,R] = ssl_rank_round(Ys); plot(R)
plot(p)

%%

fgplot(A,xy,'style','thick')