function [x,r,s] = pprpush_inverse(A, d, beta, v, tau, tol)

n = size(A,1);

% Create an iteration with the inverse system
% xs is the solution of the PageRank system
xshat = (diag(sparse(d)) - beta*A) \ ((1-beta)*v);
xs = d.*xshat; 
% 
zhat = (diag(sparse(d)) - beta*A) \ (tau*d);
z = d.*zhat;

maxit = 10*n;

% Initialize x as the solution of PageRank.
% we need to reduce it to be the solution of the 1-norm problem
x = xs;
shat = z; % shat = (eye - alpha mP')*s;
s = shat - beta*(A'*(shat./d));

for i=1:maxit
    % alg: 
    x = x - s;
    x = max(x,0);
    shat = z - (xs - x);
    s = shat - beta*(A'*(shat./d));
    r = (xs - x) - beta*(A'*((xs - x)./d));
    %x = max(x-c,0);
    fprintf('  %5i  %11.3e  %11.3e  %11.3e  %11.3e\n', i, sum(x), x'*s, min(s), sum(max(r - tau*d,0)));
    if abs(x'*s) <= sqrt(eps)
        break;
    end
end

%%

% r = (1-beta)*v;
% rp = max(r - tau*d,0);
% s = tau*d - r;
% maxit = 10*n;
% 
% % two timers, t0 is start, t1 is increment, dt1 is print increment
% t0 = tic; t1 = tic; print1 = 1; dt1 = t0 + 5;
% 
% for i=1:maxit
%     x = x + rp;
%     r = (1-beta)*v + beta*A'*(x./d) - x;
%     rp = max(r - tau*d,0);
%     s = tau*d - r;
%     dt = toc(t1);
%     if dt > dt1
%         if print1 % print the header
%             fprintf('  %5s  %11s  %11s  %11s\n', 'iter', 'sum rp', 'dual', 'sum x');
%             fprintf('  -----  -----------  -----------  -----------\n');
%         end
%         fprintf('  %5i  %11.3e  %11.3e  %11.3e\n', i, sum(rp), x'*s/n, sum(x));
%         t0 = tic;
%         print1 = 0;
%     end
%     if x'*s/n > -sqrt(eps(1)) && sum(rp) < sqrt(eps(1))
%         break;
%     end
% end



