%% Show the difference between the SSL procedures as the labeled size varies

tic
n = 150; % block size
ntrials = 1000; % 
nls = [3 5 7 10];
nerrs = [0.0 0.1];

results = zeros(numel(nls),numel(nerrs),3); % three methods
rng(1);
for nli = 1:numel(nls)
    nl = nls(nli);
    for nei = 1:numel(nerrs)
        nerr = ceil(nl*nerrs(nei))
        errs = zeros(ntrials,3);
        for i=1:ntrials
            [A,labels] = bigblock([n,n],[0.06,0.06],0.02);
            L = sparse([1:nl (nl+1):(nl+nerr) (1:nl)+n ((nl+1):(nl+nerr))+n],...
                [ones(nl,1); randi(2,nerr,1); 2*ones(nl,1); randi(2,nerr,1);], ...
                1, 2*n, 2);
            [p,Y] = joachims_ssl(A,L);
            errs(i,1) = norm(p-labels,1);
            [p,Y] = zhou_ssl_2(A,L);
            errs(i,2) = norm(p-labels,1);
            [p,Y] = zgl_ssl(A,L);
            errs(i,3) = norm(p-labels,1);
        end
        results(nli,nei,:) = mean(errs);
    end
    nl
end
toc

%%
clf;
for nei = 1:numel(nerrs)
    subplot(2,2,nei);
    cla;
    plot(nls,squeeze(results(:,nei,:)),'.-');
    xlabel('number of labels');
    ylabel('number of mistakes');
    ylim([0,200]);
end
legend('Joachims','Zhou','ZGL');
legend boxoff;


%% 
% save figures
for nei = 1:numel(nerrs)
    clf;
    plot(nls,squeeze(results(:,nei,:)),'.-','MarkerSize',12,'LineWidth',1);
    xlabel('number of labels');
    ylabel('number of mistakes');
    ylim([0,150]);
    xlim([2,11]);
    if nei==2, legend('Joachims','Zhou','ZGL','Location','South'); legend boxoff; end
    box off;
    set_figure_size([2,2]);
    print(sprintf('sparse-ssl-perf-err-%i.eps',nerrs(nei)*10),'-depsc2');
end

%%
% move figures to positions
!cp sparse-ssl-perf-err-*.eps ~/Dropbox/publications/alg-anti-diff-icml/robust-diffusion-paper/Try4-dgleich/figures/

