%% This experiment is designed to demonstrate how Joachims SSL is better
%  when the graph has some nice structures.
nl = 2;
n = 50;
[A,labels] = bigblock([n,n],[0.35,0.35],0.01);
spy(A)

L = sparse([1:nl (1:nl)+n],[ones(nl-1,1); 2; 2*ones(nl-1,1); 1], 1, 2*n, 2);

%%
[p,Y] = joachims_ssl(A,L);
norm(p-labels,1)
%%
[p,Y] = zhou_ssl_2(A,L);
norm(p-labels,1)


%%


%%

n = 2000;
endpts = 15;
mid = 15;
[A,xy] = grid_graph(2*n,1);

%%
Ltrue = [ones(n,1); 2*ones(n,1)];
L = sparse([1:endpts,2*n:-1:(2*n-endpts+1)],[1*ones(endpts,1) 2*ones(endpts,1)], 1, 2*n, 2);
L = L + sparse((n-mid+1):(n+mid),randi([1,2],2*mid,1),1,2*n,2);
%%
[p,Y] = joachims_ssl(A,L);
norm(p-Ltrue,1)
%%
[p,Y] = zhou_ssl_2(A,L);
norm(p-Ltrue,1)

%Lap = laplacian(A);
%Y = (0.0101*diag(diag(Lap)) + Lap)\L;


%% With repetitions

n = 1500;
endpts = 15;
mid = 15;
ntrials = 200;
[A,xy] = grid_graph(2*n,1);
Ltrue = [ones(n,1); 2*ones(n,1)];
errs = zeros(ntrials,2);
for i=1:ntrials
    L = sparse([1:endpts,2*n:-1:(2*n-endpts+1)],[1*ones(endpts,1) 2*ones(endpts,1)], 1, 2*n, 2);
    L = L + sparse((n-mid+1):(n+mid),randi([1,2],2*mid,1),1,2*n,2);
    [p,Y] = joachims_ssl(A,L);
    errs(i,1) = norm(p-Ltrue,1);
    [p,Y] = zhou_ssl_2(A,L);
    errs(i,2) = norm(p-Ltrue,1);
end
mean(errs)

%% More realistic repetitions

n = 1500;
endpts = 25;
mid = 15;
ncorr = 25;
ntrials = 200;
[A,xy] = grid_graph(2*n,1);
Ltrue = [ones(n,1); 2*ones(n,1)];
errs = zeros(ntrials,2);
for i=1:ntrials
    Lgiven = [(1:endpts)' ones(endpts,1);
              ((n-mid-ncorr+1):(n-ncorr))' randi([1,2],mid,1);
              ((n+ncorr+1):(n+mid+ncorr))' randi([1,2],mid,1);
              ((n-ncorr+1):n)' ones(ncorr,1);
              ((n+1):(n+ncorr))' 2*ones(ncorr,1)
              ((2*n-endpts+1):2*n)' 2*ones(endpts,1)];
    L = sparse(Lgiven(:,1),Lgiven(:,2),1,2*n,2);
    [p,Y] = joachims_ssl(A,L);
    errs(i,1) = norm(p-Ltrue,1);
    [p,Y] = zhou_ssl_2(A,L);
    errs(i,2) = norm(p-Ltrue,1);
end
mean(errs)
    

%% Even More realistic repetitions
% Or not
n = 1500;
endpts = 25;
mid = 2;
ncorr = 25;
ntrials = 200;
[A,xy] = grid_graph(2*n,1);
Ltrue = [ones(n,1); 2*ones(n,1)];
errs = zeros(ntrials,2);
for i=1:ntrials
    p1 = randperm(n-endpts-ncorr);
    p1 = p1(1:mid)+endpts;
    p2 = randperm(n-endpts-ncorr);
    p2 = p2(1:mid)+endpts+n;
    Lgiven = [(1:endpts)' ones(endpts,1);
              p1' randi([1,2],mid,1);
              p2' randi([1,2],mid,1);
              ((n-ncorr+1):n)' ones(ncorr,1);
              ((n+1):(n+ncorr))' 2*ones(ncorr,1)
              ((2*n-endpts+1):2*n)' 2*ones(endpts,1)];
    L = sparse(Lgiven(:,1),Lgiven(:,2),1,2*n,2);
    [p,Y] = joachims_ssl(A,L);
    errs(i,1) = norm(p-Ltrue,1);
    [p,Y] = zhou_ssl_2(A,L);
    errs(i,2) = norm(p-Ltrue,1);
end
mean(errs)
    

%% With a different label distribution
n = 3000;
endpts = 20;
mid = 5;
ntrials = 200;
[A,xy] = grid_graph(2*n,1);
Ltrue = [ones(n,1); 2*ones(n,1)];
errs = zeros(ntrials,2);
for i=1:ntrials
    perm = randperm(2*n);
    lverts = perm(1:2*(endpts+mid)); % labeled vertices
    L = sparse(lverts,[Ltrue(lverts(1:2*endpts)); randi([1,2],2*mid,1)], 1, 2*n, 2);
    [p,Y] = joachims_ssl(A,L);
    errs(i,1) = norm(p-Ltrue,1);
    [p,Y] = zhou_ssl_2(A,L);
    errs(i,2) = norm(p-Ltrue,1);
end
mean(errs)