%% These are the final netscience figures
%%
load xy_1
%%
%S = xy(:,2) < -500; clf;
S = xy(:,1) < 25; clf;
S = S | (xy(:,1) > 145  & xy(:,2) < -350 & xy(:,1) < 150); 
si = find(S);
S(si(4)) = 0;
fgplot(A,xy,'style','thick'); hold on;
scatter(xy(S,1),xy(S,2),100,'b','filled');
%%
A = A - diag(diag(A));
Ak = eye(size(A));
k = 5;
for i=1:k
    Ak = A*Ak + eye(size(A));
end
Ak = Ak - diag(diag(Ak));

%% 
%cmap = @() colormap(1-hot(5));
cmappart = hot(9);
cmap = @() colormap(flipud(cmappart(2:6,:)));
[px,py] = gplot(A,xy);
gdraw = @() plot(px,py,'k-','LineWidth',0.4,'Color',0.55*[1,1,1]); 
sdraw = @() plot(xy(S,1),xy(S,2),'ko','MarkerSize',7);
bigmarkers = 15;
smallmarkers = 2;
figsize = [2.75,2.75];
figarea = [0,220,-600,-120];

%%
clf;
d = sum(A,2);
v1 = d.*S; v1 = v1 / sum(v1); v1 = sparse(v1);
x = (diag(d) - 0.99*A) \ (v1);
gdraw(); hold on; 
scatter(xy(:,1),xy(:,2),bigmarkers,log10(x),'filled'); cmap();
sdraw();
set_figure_size(figsize);
axis(figarea);
axis off;
hold off;
print('netscience-K2.eps','-depsc2','-painters');

%%
d = sum(A,2);
v1 = d.*S; v1 = v1 / sum(v1); v1 = sparse(v1);
x = prl1_sweep(@(kappa) prl1_fast(A,0.0101,S,d,kappa),0.4,0.35);
clf;
gdraw(); hold on;
f = x > 0;
scatter(xy(f,1),xy(f,2),bigmarkers,log10(x(f)),'filled'); cmap();
hlast = scatter(xy(~f,1),xy(~f,2),smallmarkers,0.55*[1,1,1]);
set(hlast,'MarkerEdgeColor',0.55*[1,1,1]);
sdraw();
set_figure_size(figsize);
axis(figarea);
axis off;
hold off;
print('netscience-RK2.eps','-depsc2','-painters');


%%
dk = sum(Ak,2);
vk = dk.*S; vk = vk / sum(vk); vk = sparse(vk);
x = (diag(dk) - 0.99*Ak) \ (vk);
clf;
gdraw(); hold on;
scatter(xy(:,1),xy(:,2),bigmarkers,log10(x),'filled'); cmap();
sdraw();
%text(15.0744, -135.7977, 'Reduced Errors');
set_figure_size(figsize);
axis(figarea);
axis off;
hold off;
print('netscience-K2-Ak.eps','-depsc2','-painters');

%%
dk = sum(Ak,2);
vk = dk.*S; vk = vk / sum(vk); vk = sparse(vk);
x = prl1_sweep(@(kappa) prl1_fast(Ak,0.0101,S,dk,kappa),0.4,0.35);
clf;
gdraw(); hold on;
f = x > 0;
scatter(xy(f,1),xy(f,2),bigmarkers,log10(x(f)),'filled'); cmap();
hlast = scatter(xy(~f,1),xy(~f,2),smallmarkers,[1,1,1]);
%set(hlast,'MarkerEdgeColor','k');
set(hlast,'MarkerEdgeColor',0.55*[1,1,1]);
sdraw();
set_figure_size(figsize);
axis(figarea);
axis off;
hold off;
print('netscience-RK2-Ak.eps','-depsc2','-painters');

%% Make a set of figures showing the diffusions for different values of eps.
tau = 10^(-3.5);
alpha = 0.0101;
beta = 1/(1+alpha);
d = sum(A,2);
v1 = d.*S; v1 = v1 / sum(v1); v1 = sparse(v1);
volS = sum(v1);
kappa = tau*volS/beta;
x = prl1_fast(A,0.0101,S,d,kappa);
clf;
gdraw(); hold on;
f = x > 0;
scatter(xy(f,1),xy(f,2),bigmarkers,log10(x(f)),'filled'); cmap();
hlast = scatter(xy(~f,1),xy(~f,2),smallmarkers,0.55*[1,1,1]);
set(hlast,'MarkerEdgeColor',0.55*[1,1,1]);
sdraw();
set_figure_size(figsize);
axis(figarea);
axis off;
hold off;
print(sprintf('netscience-tol-%g.eps',log10(tau)),'-depsc2','-painters');

%% Make a movie as we vary tau
mkdir movie
%%
taus = fliplr(logspace(-7,-1.5,70));
for taui = 1:numel(taus)
    tau = taus(taui);

    alpha = 0.0101;
    beta = 1/(1+alpha);
    d = sum(A,2);
    v1 = d.*S; v1 = v1 / sum(v1); v1 = sparse(v1);
    volS = sum(v1);
    kappa = tau*volS/beta;
    x = prl1_fast(A,0.0101,S,d,kappa);
    clf;
    gdraw(); hold on;
    f = x > 0;
    scatter(xy(f,1),xy(f,2),bigmarkers,log10(x(f)),'filled'); cmap();
    hlast = scatter(xy(~f,1),xy(~f,2),smallmarkers,0.55*[1,1,1]);
    set(hlast,'MarkerEdgeColor',0.55*[1,1,1]);
    sdraw();
    set_figure_size(figsize);
    axis(figarea);
    axis off;
    hold off;
    caxis([-2.7645   -0.8412]);
    title(sprintf('\\epsilon = %g',tau));
    print(sprintf('movie/netscience-tol-%04i.png',taui),'-r200','-dpng');
end


%%
tau = 10^(-4);
alpha = 0.0101;
beta = 1/(1+alpha);
dk = sum(Ak,2);
vk = dk.*S; vk = vk / sum(vk); vk = sparse(vk);
volS = sum(vk);
kappa = tau*volS/beta;
x = prl1_fast(Ak,0.0101,S,dk,kappa);
clf;
gdraw(); hold on;
f = x > 0;
scatter(xy(f,1),xy(f,2),bigmarkers,log10(x(f)),'filled'); cmap();
hlast = scatter(xy(~f,1),xy(~f,2),smallmarkers,0.55*[1,1,1]);
set(hlast,'MarkerEdgeColor',0.55*[1,1,1]);
sdraw();
set_figure_size(figsize);
axis(figarea);
axis off;
hold off;
print(sprintf('netscience-Ak-tol-%g.eps',log10(tau)),'-depsc2','-painters');
