function [Preds,test_labels,classes] = experiment_usps_roc(digits,rad,nlabels) 
%% Use the USPS data for semi-supervised learning
% This experiment replicates the one done in Zhou 2003 on semi-supervised
% learning on a graph.

%% Load the data
mainpath = fileparts(which('load_graph'));
load(fullfile(mainpath, 'data', 'usps.mat'));

%% digit subset
patterns = [];
labels = [];
for d = digits
    filt = train_labels(d,:)==1;
    labels = [labels d*ones(1,sum(filt))];
    patterns = [patterns train_patterns(:,filt)];
    filt = test_labels(d,:)==1;
    labels = [labels d*ones(1,sum(filt))];
    patterns = [patterns test_patterns(:,filt)];
end
reallabels = labels;
[~,~,labels] = unique(reallabels);

%% construct the graph
D = pdist2(patterns',patterns','euclidean');
K = exp(-D.^2/(2*rad^2));
Gf = K - diag(diag(K));
G = Gf;
%G(G<exp(-10)) = 0; % determined this threshold by hand
G = sparse(G);
n = size(K,1);
d = sum(G,2);
% Patch up the empty rows with self-loops
%G(d==0,d==0) = eye(sum(d==0));
% Patch up empty rows with full sets (this seems critical to the last bit
% of performance)
G(:,d==0) = Gf(:,d==0);
G(d==0,:) = Gf(d==0,:);
%%
d = sum(G,2);
W = diag(1./sqrt(d))*G*diag(1./sqrt(d));
dw = sum(W,2);
%%
beta = 0.99;
alpha = (1-beta)/beta;
% R = chol(K) => R'*R = K, so Kx = Y => R'*Rx = Y => Rx = (R'\Y)
K = full(eye(n) - beta*W); R = chol(K); iK1 = @(Y) R \ (R' \ Y); 
K = full(diag(d) - beta*G); R = chol(K); iK2 = @(Y) R \ (R' \ Y); 
K = full(diag(dw) - beta*W); R = chol(K); iK3 = @(Y) R \ (R' \ Y); 
%%
classes = unique(labels);
nclasses = numel(classes);

assert(numel(labels) == n);

p = randperm(n);
train_set = zeros(n,1);
pl = labels(p);
[cp,ia] = unique(pl,'first'); %pl(ia) = c
train_set(p(ia)) = 1;
test_set = ~train_set;
test_labels = labels(test_set)';

test_filt = @(x) x(test_set);

p(ia) = [];
train_set(p(1:(nlabels - nclasses))) = 1;

Y = full(sparse(find(train_set),labels(logical(train_set)),1,n,nclasses));

PredMat = zeros(numel(test_labels),nclasses);
Preds.K1 = PredMat;
Preds.K2 = PredMat;
Preds.K3 = PredMat;
Preds.RK3 = PredMat;
for c=1:nclasses
    y = Y(:,c);
    s = Y(:,c);

    Preds.K1(:,c) = test_filt(iK1(y));
    Preds.K2(:,c) = test_filt(iK2(y));
    Preds.K3(:,c) = test_filt(iK3(y));
    Preds.RK3(:,c) = test_filt(prl1_push(W,alpha,(s>0).*dw,dw,1e-5));
end
