%% In this script we wish to study the usps sparse_dense problem
% to determine appropriate nn cutoffs 
%
% summary:
%  nn=3 => avgdeg = 4.57    ... k=2 => 18.12    ... k=3 => 52.34 ... k=4 => 127
%  nn=5 => avgdeg = 7.47    ... k=2 => 39.18    ... k=3 => 137.8
%  nn=10 => avgdeg = 14.7   ... k=2 => 103.83   ... k=3 => 404.0
%  nn=13 => avgdeg = 19.0
%  nn=25 => avgdeg = 35.2   ... k=2 => 322.64
%  nn=28 => avgdeg = 40.5
%  nn=37 => avgdeg = 53.3
%  nn=73 => avgdeg = 104.4
%
% So we have the following pairs
%  nn=13, nn=3,k=2 => avgdeg ~18
%  nn=28, nn=5,k=2 => avgdeg ~39
%  nn=37, nn=3,k=3 => avgdeg ~52
%  nn=73, nn=10,k=2 => avgdeg ~103
%  nn=97, nn=5,k=3 => avgdeg ~137
%  
%  nn=230, nn=25,k=2 => avgdeg ~322
%
% Strange, these seem different than what I had before. 
%   Oh, it was a weighted degree vs. nnz thing. Using nnz was correct.

digits = 1:10;
rad = 2.5;
%%
mainpath = fileparts(which('load_graph'));
load(fullfile(mainpath, 'data', 'usps.mat'));
%% digit subset
patterns = [];
labels = [];
for d = digits
    filt = train_labels(d,:)==1;
    labels = [labels d*ones(1,sum(filt))];
    patterns = [patterns train_patterns(:,filt)];
    filt = test_labels(d,:)==1;
    labels = [labels d*ones(1,sum(filt))];
    patterns = [patterns test_patterns(:,filt)];
end
reallabels = labels;
[~,~,labels] = unique(reallabels);
%%
Dist = pdist2(patterns',patterns','euclidean');
K = exp(-Dist.^2/(2*rad^2));
G = K - diag(diag(K)); % remove diagonal
G(G<sqrt(eps(1))) = 0;
df = sum(G,2) >= sqrt(eps(1));
G = G(df,df);

%%
nn = 3;
G1 = spones(sparsify_large(G,'nearest',nn));
fprintf('G1 - avgdeg - %f\n', full(mean(sum(G1))))

kpow = 4;
Gk = G1;
Gk = Gk + speye(size(G,1));
for i=2:kpow
    Gk = G1*Gk + speye(size(G,1));
end
Gk = Gk - diag(diag(Gk));
% Note we have to be careful with how we compute this one because it's weighted        
fprintf('Gk - avgdeg - %f\n', full(nnz(Gk)/size(Gk,1)));

%% Check what I had before
digits = 1:10;
nn = 5; k=1;
data = usps_setup_sparse_dense(digits,nn,k);
nn = 5; k=2;
data = usps_setup_sparse_dense(digits,nn,k);
nn = 30; k=1;