%% Make sure our SSL formulations are equivalent
% If we set kappa = 0, then the regularized solutions should be identical
% to RK2 and RK3
digits = [2:4];
nlabels = 10;
rad = 2.5;
kappa = 0;

data = usps_setup(digits,rad);
struct2vars(data);
classes = unique(labels);
nclasses = numel(classes);

%%
rng(1);

%%
[test_set,test_labels,train_set,train_labels] = build_test_train(labels,nlabels);
Y = full(sparse(find(train_set),train_labels,1,n,nclasses));

PredMat = zeros(n,nclasses);
Preds.K2 = iK2(Y);
Preds.K3 = iK3(Y);
Preds.RK3 = PredMat;
Preds.RK2 = PredMat;
for c=1:nclasses
    s = Y(:,c);
    
    Preds.RK2(:,c) = prl1_custom(G,alpha,(s>0)./d,d,kappa,prsolverG);
    
    Preds.RK3(:,c) = prl1_custom(W,alpha,(s>0)./dw,dw,kappa,prsolverW);
end

%%
assert(norm(Preds.K2-Preds.RK2*1/(1-beta),'fro') < 1e-12);
assert(norm(Preds.K3-Preds.RK3*1/(1-beta),'fro') < 1e-12);

%%
[test_set,test_labels,train_set,train_labels] = build_test_train(labels,nlabels);
Y = full(sparse(find(train_set),train_labels,1,n,nclasses));

PredMat = zeros(n,nclasses);
Preds.K2 = iK2(diag(d)*Y);
Preds.K3 = iK3(diag(dw)*Y);
Preds.RK3 = PredMat;
Preds.RK2 = PredMat;
for c=1:nclasses
    s = Y(:,c);
    
    Preds.RK2(:,c) = prl1_custom(G,alpha,(s>0),d,kappa,prsolverG);
    
    Preds.RK3(:,c) = prl1_custom(W,alpha,(s>0),dw,kappa,prsolverW);
end

%%
assert(norm(Preds.K2-Preds.RK2*1/(1-beta),'fro') < 1e-12);
assert(norm(Preds.K3-Preds.RK3*1/(1-beta),'fro') < 1e-12);

