function errrate = usps_errrate(digits,rad,nlabeled,ntrials,nerrorslist) 
%% Use the USPS data for semi-supervised learning
% This experiment replicates the one done in Zhou 2003 on semi-supervised
% learning on a graph.

if ~exist('nerrorslist','var') || isempty(nerrorslist)
    nerrorslist = zeros(numel(nlabeled),1); 
end

assert(numel(nlabeled) == numel(nerrorslist))

data = usps_setup(digits,rad);


% struct2vars(data); % nice slick function, but it doesn't work with
% parfor!
G = data.G;
n = data.n;
d = data.d;
W = data.W;
dw = data.dw;
iK1 = data.iK1;
iK2 = data.iK2;
iK3 = data.iK3;
prsolverG = data.prsolverG;
prsolverW = data.prsolverW;
labels = data.labels;
nmissing = data.nmissing;
alpha = data.alpha;


%%
nl = numel(nlabeled);
classes = unique(labels);
nclasses = numel(classes);

assert(numel(labels) == n);
errrate = zeros(nl, ntrials, 6, 3);
%%
for j=1:nl
    nlabels = nlabeled(j)*nclasses;
    nerrors = nerrorslist(j);
    fprintf('Now testing with %i labels\n', nlabels);
    
    for i=1:ntrials
        curerrs = zeros(6,3);
        
        %%
        % pick the training set, it needs to have one element from each class.
        t0 = tic;
        
        [test_set,test_labels,train_set,train_labels] = build_test_train(labels,nlabels,nerrors);
        Y = full(sparse(find(train_set),train_labels,1,n,nclasses));
        
        Pred = iK1(Y);
        curerrs(1,:) = usps_score(Pred,test_set,test_labels,G,nmissing);
        
        Pred = iK2(repmat(d,1,nclasses).*Y);
        curerrs(2,:) = usps_score(Pred,test_set,test_labels,G,nmissing);
        
        Pred = iK3(repmat(dw,1,nclasses).*Y);
        curerrs(3,:) = usps_score(Pred,test_set,test_labels,G,nmissing);
        
        % Predict using cuts
        Pred = zeros(size(Y));
        for c=1:nclasses
            s = Y(:,c);
            assert(all(s >= 0));
            
            %tic
            fprintf('Solving with ACL Cut nl = %i trial = %i, class = %i\n', nlabels, i, c);
            prl1 = @(kappa) prl1_custom(W,alpha,(s>0),dw,kappa,prsolverW);
            xt = prl1_sweep(prl1,0.75,0.5);
            Pred(:,c) = xt; % doesn't actually change results, but "right"
        end
        curerrs(4,:) = usps_score(Pred,test_set,test_labels,G,nmissing);
        
        % Joachims
        [~,Pred] = joachims_ssl_fast(G,d,Y);
        curerrs(5,:) = usps_score(Pred,test_set,test_labels,G,nmissing);

        % PR SSL
        Pred = zeros(size(Y));
        for c=1:nclasses
            s = Y(:,c);
            assert(all(s >= 0));
            
            %tic
            fprintf('Solving with ACL Cut nl = %i trial = %i, class = %i\n', nlabels, i, c);
            prl1 = @(kappa) prl1_custom(G,alpha,(s>0),d,kappa,prsolverG);
            xt = prl1_sweep(prl1,0.75,0.5);
            Pred(:,c) = xt; % doesn't actually change results, but "right"
        end
        curerrs(6,:) = usps_score(Pred,test_set,test_labels,G,nmissing);
        
        errrate(j,i,:,:) = curerrs;

        fprintf('errrates(%2i) = ', nlabels);
        fprintf('%10.5f ', curerrs(:,1));
        fprintf('\n');
        fprintf('errrates(%2i) = ', nlabels);
        fprintf('%10.5f ', curerrs(:,2));
        fprintf('\n');
        fprintf('errrates(%2i) = ', nlabels);
        fprintf('%10.5f ', curerrs(:,3));
        fprintf('\n');
        fprintf('  trial time = %f\n', toc(t0));
    end
end
%%
