function errrate = usps_errrates_unify(data,nlabeled,ntrials,nerrorslist,varargin) 
%% Use the USPS data for semi-supervised learning
% This experiment replicates the one done in Zhou 2003 on semi-supervised
% learning on a graph.

parser = inputParser;
parser.addOptional('useparfor',false);
parser.addOptional('regsize',[0.5,0.33]);
parser.parse(varargin{:})

useparfor = parser.Results.useparfor;
regsize = parser.Results.regsize;

if ~exist('nerrorslist','var') || isempty(nerrorslist)
    nerrorslist = zeros(numel(nlabeled),1); 
end

assert(numel(nlabeled) == numel(nerrorslist))

%data = usps_setup_sparse_dense(digits,nn,k);
if useparfor
    save('temp_usps_parfor.mat','-v7.3','-struct','data');
end
labels = data.labels;
alpha = data.alpha;
n = data.n;
d = data.d;
nmissing = data.nmissing;
if useparfor
    clear data;
    datasave = [];
else
    datasave = data;
end

%%
nl = numel(nlabeled);
classes = unique(labels);
nclasses = numel(classes);

assert(numel(labels) == n);
errrate = zeros(nl, ntrials, 6, 3);
%%
for j=1:nl
    nlabels = nlabeled(j)*nclasses;
    nerrors = nerrorslist(j);
    fprintf('Now testing with %i labels\n', nlabels);
    
    parfor i=1:ntrials
        if useparfor
            data = load('temp_usps_parfor.mat');
        else
            data = datasave;
        end
        curerrs = zeros(6,3);
        
        %%
        % pick the training set, it needs to have one element from each class.
        t0 = tic;
        
        [test_set,test_labels,train_set,train_labels] = build_test_train(labels,nlabels,nerrors);
        Y = full(sparse(find(train_set),train_labels,1,n,nclasses));
        
        % row-labels are left-overs
        Pred = data.iK2(repmat(d,1,nclasses).*Y);
        curerrs(2,:) = usps_score(Pred,test_set,test_labels,data.G,nmissing);
        
        % PR SSL
        Pred = zeros(size(Y));
        for c=1:nclasses
            s = Y(:,c);
            assert(all(s >= 0));
            
            %tic
            fprintf('Solving with ACL Cut nl = %i trial = %i, class = %i\n', nlabels, i, c);
            prl1 = @(kappa) prl1_custom(data.G,alpha,(s>0),data.d,kappa,data.prsolverG);
            xt = prl1_sweep(prl1,regsize(1),regsize(2));
            Pred(:,c) = xt; % doesn't actually change results, but "right"
        end
        curerrs(6,:) = usps_score(Pred,test_set,test_labels,data.G,nmissing);
        
        errrate(j,i,:,:) = curerrs;

        fprintf('errrates(%2i) = ', nlabels);
        fprintf('%10.5f ', curerrs(:,1));
        fprintf('\n');
        fprintf('errrates(%2i) = ', nlabels);
        fprintf('%10.5f ', curerrs(:,2));
        fprintf('\n');
        fprintf('errrates(%2i) = ', nlabels);
        fprintf('%10.5f ', curerrs(:,3));
        fprintf('\n');
        fprintf('  trial time = %f\n', toc(t0));
    end
end
%%
