function data = usps_setup_sparse_dense(digits, nn, k, beta)

%% default value of beta
if nargin < 4
    beta = 0.99;
end

if numel(nn) == 2
    nnerr = nn(2);
    nn = nn(1);
else
    nnerr = 0;
end

%% Load the data
mainpath = fileparts(which('load_graph'));
load(fullfile(mainpath, 'data', 'usps.mat'));

%% digit subset
patterns = [];
labels = [];
for d = digits
    filt = train_labels(d,:)==1;
    labels = [labels d*ones(1,sum(filt))];
    patterns = [patterns train_patterns(:,filt)];
    filt = test_labels(d,:)==1;
    labels = [labels d*ones(1,sum(filt))];
    patterns = [patterns test_patterns(:,filt)];
end
reallabels = labels;
[~,~,labels] = unique(reallabels);

%% construct the graph
Dist = pdist2(patterns',patterns','euclidean');
K = exp(-Dist.^2/(2*2.5^2));
G = K - diag(diag(K)); % remove diagonal
% the next steps are something I'd rather exclude
G(G<sqrt(eps(1))) = 0;
df = sum(G,2) >= sqrt(eps(1));
G = G(df,df);
labels = labels(df);
nmissing = size(K,1) - sum(df);
%% sparsify the graph
G = spones(sparsify_large(G,'nearest',nn));
%% add in errors
if nnerr > 0
    Gerr = sprand(size(G,1),size(G,1),nnerr/size(G,1));
    Gerr = spones(triu(Gerr,1));
    Gerr = Gerr | Gerr';
    G = spones(G+Gerr);
    G = G - diag(diag(G));
end
%% densify the graph
if k>1
    G1 = G;
    G = G1 + speye(size(G,1));
    for i=2:k
        G = G1*G + speye(size(G,1));
    end
    G = G - diag(diag(G));
end
fprintf('Final average degree: %g\n', nnz(G)/size(G,1));
%%
n = size(G,1);
d = sum(G,2);
W = diag(sparse(1./sqrt(d)))*G*diag(sparse(1./sqrt(d)));
P = G*diag(sparse(1./d));
dw = sum(W,2);
%%
alpha = (1-beta)/beta;
iK1 = [];
K = full(diag(d) - beta*G); R = chol(K); iK2 = @(Y) R \ (R' \ Y); 
iK3 = [];

prsolverG = @(A,d,beta,B) repmat(d,1,size(B,2)).*iK2(B); % d here is an input
prsolverW = []; 

%% Save output
data.G = G;
data.n = n;
data.d = d;
data.P = P;
data.W = W;
data.dw = dw;
data.iK1 = iK1;
data.iK2 = iK2;
data.iK3 = iK3;
data.prsolverG = prsolverG;
data.prsolverW = prsolverW;
data.labels = labels;
data.nmissing = nmissing;
data.patterns = patterns;
data.beta = beta;
data.alpha = alpha;
