function Y = generic_ssl(A,s,t,varargin)
% GENERIC_SSL 
% 
%
% node_weights 
% pin_negative
% pin_positive
%
% Inputs:
%   Zhou K2
%      generic_ssl(A,alpha*pos,alpha*d.*all-pos)
%      alpha = (1-beta)/beta;
%      weight of alpha*d.*pos connecting source s to graph
%      weight of alpha*d.*(all - pos) connecting sink t to graph
%      this is the solution of
%      (D - beta*A) \ pos ( or (L + alpha*A) \ pos )
% 
%   Zhou K1
%      generic_ssl(A,alpha*d.*pos,alpha*d.*(all-pos),'node_weights',sqrt(d))
%      alpha = (1-beta)/beta;
%      weight of alpha*d.*pos connecting source s to graph
%      weight of alpha*d.*(all - pos) connecting sink t to graph
%      pos scaled by sqrt(d) before the solution
%      
%   Joachims
%      generic_ssl(A,pos,labeled-pos)
%      
%   Weighted Joachims
%      generic_ssl(A,d.*pos,d.*labeled-pos)
%
%   ZGL 
%      generic_ssl(A,pos,labeled-pos,'pin_positive',true,'pin_negative',true);
%      

parser = inputParser;
parser.addParamValue('node_weights',[]);
parser.addParamValue('pin_positive',false,@islogical);
parser.addParamValue('pin_negative',false,@islogical);
parser.parse(varargin{:});
w = parser.Results.node_weights;


n = size(A,1);
Y = zeros(n,1);

d = sum(A,2);

K = diag(d) - A + diag(s) + diag(t);
b = s;

if ~isempty(w)
    b = w.*s;
end

Y = zeros(n,1);
subset = true(n,1);
filter = false;
if parser.Results.pin_positive
    Y(s>0) = 1;
    subset(s>0) = 0;
    filter = true;
end

if parser.Results.pin_negative
    Y(t>0) = 0;
    subset(t>0) = 0;
    filter = true;
end

if filter
    b = -K*b;
    Y(subset) = K(subset,subset) \ b(subset);
else
    Y = K \ b;
end

if ~isempty(w)
    Y = w.*Y;
end