function L = pearson_confidence(C,N,coeff)
% PEARSON_CONFIDENCE Confidence levels of each positive correlation.
%
% L = pearson_confidence(C,N) takes in the correlation coeffiecients
% as well as the number of measurements for each correlation coefficient
% and returns the level at which the entry is positively correlated.
%
% L = pearson_confidence(C,N, coeff) gives us the confidence level at which
% the correlation exceeds coeff. By default coeff is 0, so positive
% correlations.

if ~exist('coeff','var') || isempty(coeff), coeff = 0.; end

% compute column-wise, because otherwise it takes too much time!
L = zeros(size(C));
for i=1:size(C,2)
    ci = C(:,i);
    ni = N(:,i);
    z = atanh(ci);
    conf = atanh(coeff)-z.*sqrt(ni-3).*(ni > 3);
    l = min((1-normcdf(conf))*2, 2*normcdf(conf));
    L(:,i) = l;
end


% % compute element-wise because otherwise this takes too much memory.
% L = zeros(size(C));
% for i=1:numel(C)
%     if N(i) < 4
%         L(i) = 1;
%     else
%         z = atanh(C(i));
%         conf = -z.*sqrt(N(i)-3);
%         l = min((1-normcdf(conf))*2, 2*normcdf(conf));
%         L(i) = l;
%     end
% end
% Z = atanh(C);
% we need to find the confidence level that excludes 0 
% Z + X./sqrt(N-3) = 
% Conf = -Z .* sqrt(N-3);
% L = min((1-normcdf(Conf))*2, 2*normcdf(Conf));
