function x = prl1_cvx(A,alpha,s,d,kappa)
% PRL1_CVX Solve the PRL1 problem using the CVX algorithm.
% 
% This code solves the following problem:
%
% cvx_begin
%    variable z(n);
%    minimize 1/2*quad_form(z,Q) - alpha*sum(z.*d.*s) + kappa*sum(d.*z)
%    subject to
%        z >= 0;
% cvx_end
%
% where D = diag(sum(A,2)) (diagonal matrix of row-sums or weighted degrees)
%       L = D - A          (Laplacian matrix of the graph)
%       Q = alpha*D + L    (PageRank Laplacian)
%
% Via the cvx directly method. 

n = size(A,1);
Q = alpha*diag(d) + (diag(d) - A);
cvx_begin quiet 
    variable x(n);
    minimize 1/2*quad_form(x,Q) - alpha*sum(x.*d.*s) + kappa*sum(d.*x)
    subject to
        x >= 0;
cvx_end




            