function z = prl1_push(A,alpha,s,d,kappa)
% PRL1_PUSH Solve the PRL1 problem using the Push algorithm.
% 
% z = prl1_push(A,alpha,s,d,kappa)
%
% This code solves the following problem:
%
% cvx_begin
%    variable z(n);
%    minimize 1/2*quad_form(z,Q) - alpha*sum(z.*d.*s) + kappa*sum(d.*z)
%    subject to
%        z >= 0;
% cvx_end
%
% where D = diag(sum(A,2)) (diagonal matrix of row-sums or weighted degrees)
%       L = D - A          (Laplacian matrix of the graph)
%       Q = alpha*D + L    (PageRank Laplacian)
%
% Via the PageRank push method. This involves translating it into a
% different normalization, solving that normalization via the push
% algorithm (really just, Gauss-Seidel or coordinate descent) and then
% reverts to the normaliztion applied above

beta = 1/(1+alpha);
volS = sum(d.*s);
v = (d.*s)/volS;
v = sparse(v);
d = full(d);
A = sparse(A);

tau = kappa*beta/volS;

% it's the last return because it isn't usually do this!
[~,~,~,~,x] = pprpush_weighted_mex(A, d, v, tau, beta, sqrt(eps(1)));
%x = acl_method_1(normout(A), (1-beta)*v, d, beta, tau);    


z = (x./d)*volS;



            
