[A,xy] = load_graph('four-clusters');
n = size(A,1);
c = 4;
labels = [1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 3 3 3 3 3 3 3 4 4 4 4 4];
L = zeros(n,c);
L(1,1) = 1;
L(11,2) = 1;
L(18,3) = 1;
L(26,4) = 1;

all = ones(n,1);
l = sum(L,2);

assert(~any(diag(A)~=0));
assert(isequal(A,A'))

d = sum(A,2);

%% test Zhou's algorithm 2
% Zhou's algorithm 2 is to solve
%   (D - beta*A) \ L
% where 
beta = 0.99;
alpha = (1-beta)/beta;
Y1 = (diag(d) - beta*A) \ L;
Y2 = zeros(size(L));
for i=1:c
    pos = L(:,i);
    Y2(:,i) = generic_ssl(A,alpha*pos,alpha*(d.*(all) - pos));
    % This has the vector scale differently
end
Y2 = (1/(1-beta))*Y2;
assert(norm(Y1 - Y2,'fro') < 1e-8);

%%
[~,Y2] = zhou_ssl_2(A,L,beta);
assert(norm(Y1 - Y2,'fro') < 1e-8);

%% test Zhou's algorithm 1
% Zhou's algorithm 1 is to solve
%   (eye(n) - beta*D^{-1/2}*A*D^{-1/2}) \ L
% where 
beta = 0.99;
alpha = (1-beta)/beta;
Y1 = (eye(n) - beta*wadjacency(A)) \ L;
Y2 = zeros(size(L));
for i=1:c
    pos = L(:,i);
    Y2(:,i) = generic_ssl(A,alpha*pos,alpha*(d.*(all) - pos),'node_weights',sqrt(d));
    % This has the vector scale differently
end
Y2 = (1/(1-beta))*Y2;
assert(norm(Y1 - Y2,'fro') < 1e-8);
%%
[~,Y2] = zhou_ssl_1(A,L,beta);
assert(norm(Y1 - Y2,'fro') < 1e-8);

%% test Zhou's algorithm 1p
% Zhou's algorithm 1' is to solve
%  (eye - beta*D^{-1} A) \ L
beta = 0.99;
alpha = (1-beta)/beta;
Y1 = (eye(n) - beta*diag(1./d)*(A)) \ L;
Y2 = zeros(size(L));
for i=1:c
    pos = L(:,i).*d;
    Y2(:,i) = generic_ssl(A,alpha*pos,alpha*(d.*(all) - pos));
    % This has the vector scale differently
end
Y2 = (1/(1-beta))*Y2;
assert(norm(Y1 - Y2,'fro') < 1e-8);
%%
[~,Y2] = zhou_ssl_1p(A,L,beta);
assert(norm(Y1-Y2,'fro') < 1e-8);

%% Test Joachim's method
Y1 = (diag(d) - A + diag(l)) \ L;
for i=1:c
    pos = L(:,i);
    neg = l - pos;
    Y2(:,i) = generic_ssl(A,pos,neg);
    % This has the vector scale differently
end
assert(norm(Y1 - Y2,'fro') < 1e-8);
%%
[~,Y2] = joachims_ssl(A,L);
assert(norm(Y1 - Y2,'fro') < 1e-8);

%% Test Joachim's method 2
Y1 = (diag(d) - A + diag(l.*d)) \ L;
for i=1:c
    pos = L(:,i);
    neg = l - pos;
    Y2(:,i) = generic_ssl(A,pos,d.*l - pos);
    % This has the vector scale differently
end
assert(norm(Y1 - Y2,'fro') < 1e-8);

%% Test ZGL
% ZGL is like Joachim's, but it pins the vertices to each side
F = diag(d) - A;
K = F(~l,~l);
% assemble the RHS
B = -F*L;
Y1 = zeros(size(L));
Y1(l>0,:) = L(l>0,:); % apply labeled rows
Y1(~l,:) = K \ B(~l,:); % fill in missing rows

for i=1:c
    pos = L(:,i);
    neg = l - pos;
    Y2(:,i) = generic_ssl(A,pos,neg,'pin_positive',true,'pin_negative',true);
    % This has the vector scale differently
end
assert(norm(Y1 - Y2,'fro') < 1e-8);
%%
[~,Y2] = zgl_ssl(A,L);
assert(norm(Y1 - Y2,'fro') < 1e-8);
