%% Test a construction that seems to make pprpush run forever
% Also we used this example to compare between our push code
% and gurobi 

digits = [2,3,4,5];
rad = 2.5;
nlabeled = 20;
ntrials = 1;
kappa = 1e-4;


%% Load the data
mainpath = fileparts(which('load_graph'));
load(fullfile(mainpath, 'data', 'usps.mat'));

%% digit subset
patterns = [];
labels = [];
for d = digits
    filt = train_labels(d,:)==1;
    labels = [labels d*ones(1,sum(filt))];
    patterns = [patterns train_patterns(:,filt)];
    filt = test_labels(d,:)==1;
    labels = [labels d*ones(1,sum(filt))];
    patterns = [patterns test_patterns(:,filt)];
end
reallabels = labels;
[~,~,labels] = unique(reallabels);

%% construct the graph
Dist = pdist2(patterns',patterns','euclidean');
K = exp(-Dist.^2/(2*rad^2));
G = K - diag(diag(K)); % remove diagonal

%%
n = size(G,1);
d = sum(G,2);
W = diag(1./sqrt(d))*G*diag(1./sqrt(d));
P = G*diag(1./d);
dw = sum(W,2);
beta = 0.99;
alpha = (1-beta)/beta;

%%
nl = numel(nlabeled);
classes = unique(labels);
nclasses = numel(classes);

assert(numel(labels) == n);
errrate = zeros(nl, ntrials, 4);
for j=1:nl
    nlabels = nlabeled(j);
    fprintf('Now testing with %i labels\n', nlabels);
    for i=1:ntrials
        %%
        % pick the training set, it needs to have one element from each class.
        p = randperm(n);
        train_set = zeros(n,1);
        pl = labels(p);
        [cp,ia] = unique(pl,'first'); %pl(ia) = c
        train_set(p(ia)) = 1;
        test_set = ~train_set;
        test_labels = labels(test_set)';

        p(ia) = [];
        train_set(p(1:(nlabels - nclasses))) = 1;

        Y = full(sparse(find(train_set),labels(logical(train_set)),1,n,nclasses));
        Yt = diag(1./sqrt(d))*Y; 
        
        Pred = zeros(size(Y));
        for c=1:nclasses
            s = Yt(:,c);
            assert(all(s >= 0));
            
            tic
            fprintf('Solving with ACL Cut i=%i\n', i);
            xt = prl1_fast(W,alpha,(s>0).*dw,dw,kappa);
            toc
            
            % Removed because we don't want this normalization now
            % We removed the degree weighting here.
            Pred(:,c) = xt; % doesn't actually change results, but "right"
        end
        
        PredR = zeros(n,nclasses);
        for c=1:nclasses
            [~,perm] = sort(Pred(:,c),'descend');
            PredR(perm,c) = 1:n;
        end
        [~,Pi] = min(PredR,[],2); % find the largest entry in each row
        pred_labels = Pi(test_set);
        errs = numel(test_set) - sum(pred_labels == test_labels);
        errrate(j,i,3) = errs/numel(test_set);
        
        
        % Predict using cuts
        Pred = zeros(size(Y));
        for c=1:nclasses
            s = Yt(:,c);
            assert(all(s >= 0));
            
            tic
            fprintf('Solving with ACL Cut i=%i\n', i);
            %xt = prl1_gurobi(W,alpha,(s>0).*dw,dw,kappa);
            toc
            
            % Removed because we don't want this normalization now
            % We removed the degree weighting here.
            Pred(:,c) = xt; % doesn't actually change results, but "right"
        end
        
        PredR = zeros(n,nclasses);
        for c=1:nclasses
            [~,perm] = sort(Pred(:,c),'descend');
            PredR(perm,c) = 1:n;
        end
        [~,Pi] = min(PredR,[],2); % find the largest entry in each row
        pred_labels = Pi(test_set);
        errs = numel(test_set) - sum(pred_labels == test_labels);
        errrate(j,i,4) = errs/numel(test_set);
        
        fprintf('errrates(%2i) = ', nlabels);
        fprintf('%10.5f ', errrate(j,i,:));
        fprintf('\n');
    end
end
%%
