%% Make a few different figures from the USPS dataset for a talk

%% Load the data
load data/usps.mat
solve_cvx = 0; % = 0 means to use gurobi, = 1 means use (SLOW) cvx



%% digit subset
digits = [1,2,3,4]+1; % handle the offset that 1 = 0
patterns = [];
labels = [];
for d = digits
    filt = train_labels(d,:)==1;
    labels = [labels d*ones(1,sum(filt))];
    patterns = [patterns train_patterns(:,filt)];
    filt = test_labels(d,:)==1;
    labels = [labels d*ones(1,sum(filt))];
    patterns = [patterns test_patterns(:,filt)];
end
labels = labels-1; % handle the offset 

%% construct the graph and output it
D = pdist2(patterns',patterns','euclidean');
rad = 1.25;
K = exp(-D.^2/(2*rad^2));
Gf = K - diag(diag(K));
G = Gf;
%G(G<exp(-10)) = 0; % determined this threshold by hand
% G = G;
n = size(K,1);
d = sum(G,2);
% Patch up the empty rows with self-loops
%G(d==0,d==0) = eye(sum(d==0));
% Patch up empty rows with full sets (this seems critical to the last bit
% of performance)
G(:,d==0) = Gf(:,d==0);
G(d==0,:) = Gf(d==0,:);
%% 
d = sum(G,2);
W = diag(1./sqrt(d))*G*diag(1./sqrt(d));
P = G*diag(1./d);
dw = sum(W,2);
%%
imagesc(log10(W+1e-12)); colormap(hot)
%%
l10 = log10(W+1e-12);
m = min(l10(:));
M = max(l10(:));
%%
image(64*(l10-m)/(M-m));
imwrite(255*(l10-m)/(M-m),hot(255),sprintf('usps-1234-sigma-1.png'));

%% construct the graph and output it
D = pdist2(patterns',patterns','euclidean');
rad = 2.5;
K = exp(-D.^2/(2*rad^2));
Gf = K - diag(diag(K));
G = Gf;
%G(G<exp(-10)) = 0; % determined this threshold by hand
% G = G;
n = size(K,1);
d = sum(G,2);
% Patch up the empty rows with self-loops
%G(d==0,d==0) = eye(sum(d==0));
% Patch up empty rows with full sets (this seems critical to the last bit
% of performance)
G(:,d==0) = Gf(:,d==0);
G(d==0,:) = Gf(d==0,:);
%% 
d = sum(G,2);
W = diag(1./sqrt(d))*G*diag(1./sqrt(d));
P = G*diag(1./d);
dw = sum(W,2);
%%
imagesc(log10(W+1e-8)); colormap(hot)
%%
l10 = log10(W+1e-8);
m = min(l10(:));
M = max(l10(:));
%%
image(64*(l10-m)/(M-m));
imwrite(255*(l10-m)/(M-m),hot(255),sprintf('usps-1234-sigma-2.png'));

%% Make a picture of the predictions for one task
d = sum(G,2);
W = diag(1./sqrt(d))*G*diag(1./sqrt(d));
P = G*diag(1./d);
dw = sum(W,2);
%%
beta = 0.99;
alpha = (1-beta)/beta;
% R = chol(K) => R'*R = K, so Kx = Y => R'*Rx = Y => Rx = (R'\Y)
K = full(eye(n) - beta*W); R = chol(K); iK1 = @(Y) R \ (R' \ Y); 
K = full(diag(d) - beta*G); R = chol(K); iK2 = @(Y) R \ (R' \ Y); 
K = full(diag(dw) - beta*W); R = chol(K); iK3 = @(Y) R \ (R' \ Y); 
%%
ntrials = 1;
nlabeled = [20];
nl = numel(nlabeled);
classes = unique(labels);
nclasses = numel(classes);

assert(numel(labels) == n);
errrate = zeros(nl, ntrials, 4);
for j=1:nl
    nlabels = nlabeled(j)
    for i=1:ntrials
        %%
        % pick the training set, it needs to have one element from each class.
        p = randperm(n);
        train_set = zeros(n,1);
        pl = labels(p);
        [cp,ia] = unique(pl,'first'); %pl(ia) = c
        train_set(p(ia)) = 1;
        test_set = ~train_set;
        test_labels = labels(test_set)';

        p(ia) = [];
        train_set(p(1:(nlabels - nclasses))) = 1;

        Y = full(sparse(find(train_set),labels(logical(train_set)),1,n,nclasses));
        Yt = diag(1./sqrt(d))*Y;      
        
        Pred1 = iK1(Y);
      
        
        Pred2 = iK2(Y);
   
        
        Pred3 = iK3(Y);
        
        
        % Predict using cuts
        Pred4 = zeros(size(Y));
        for c=1:nclasses
            s = Yt(:,c);
            assert(all(s >= 0));
            
            tau = 1e-6;
            kappa = tau*sum(d.*s)/beta;
 
            tau = 1e-4*sum(s>0);
            fprintf('Solving with ACL Cut i=%i\n', i);
            xt = prl1_gurobi(W,alpha,(s>0).*dw,dw,1e-5);

            Pred4(:,c) = sqrt(d).*xt; % doesn't actually change results, but "right"
        end
    end
end
%%
plot(Pred4);
set_figure_size([5,3]);
axis tight;
%%
print('usps-1234-sigma-2-pred-rk3.eps','-depsc2');

%% Print out a bunch of images.
rescaleim = @(x) 255*(x-min(x(:)))/(max(x(:))-min(x(:)));
writedig = @(x,str) imwrite(reshape(rescaleim(x),16,16)',bone(255),str);
%%
writedig(patterns(:,16),'usps-1-1.png');
writedig(patterns(:,36),'usps-1-2.png');
writedig(patterns(:,42),'usps-1-3.png');
writedig(patterns(:,100),'usps-1-4.png');
%%
writedig(patterns(:,1500),'usps-2-1.png');
writedig(patterns(:,1800),'usps-2-2.png');
writedig(patterns(:,1700),'usps-2-3.png');
%%
writedig(patterns(:,2300),'usps-3-1.png');
writedig(patterns(:,2400),'usps-3-2.png');
writedig(patterns(:,2450),'usps-3-3.png');
%%
writedig(patterns(:,3050),'usps-4-1.png');
writedig(patterns(:,3250),'usps-4-2.png');
writedig(patterns(:,3300),'usps-4-3.png');
%%
imagesc(reshape(patterns(:,3300),16,16)'); colormap(bone);
